@import 'partial';

@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
@media (min-width: 500px) {  div { color: red; } }
